@@ -1,6 +1,7 @@
 '''
 This file is part of RTSLib.
 Copyright (c) 2011-2013 by Datera, Inc
+Copyright (c) 2011-2014 by Red Hat, Inc.
 
 Licensed under the Apache License, Version 2.0 (the "License"); you may
 not use this file except in compliance with the License. You may obtain
@@ -23,7 +24,7 @@ from target import NodeACL, NetworkPortal, TPG, Target
 from fabric import FabricModule
 
 from tcm import FileIOStorageObject, BlockStorageObject
-from tcm import PSCSIStorageObject, RDMCPStorageObject
+from tcm import PSCSIStorageObject, RDMCPStorageObject, UserBackedStorageObject
 from tcm import StorageObjectFactory
 
 __version__ = 'GIT_VERSION'
