@@ -1,7 +1,7 @@
 '''
 This file is part of RTSLib.
 Copyright (c) 2011-2013 by Datera, Inc
-Copyright (c) 2013 by Andy Grover
+Copyright (c) 2011-2014 by Red Hat, Inc.
 
 Licensed under the Apache License, Version 2.0 (the "License"); you may
 not use this file except in compliance with the License. You may obtain
@@ -176,7 +176,7 @@ class _BaseFabricModule(CFSNode):
                 if os.path.isfile(path):
                     return fread(path)
             else:
-                raise RTSLibError("Can't find version for fabric module %s."
+                raise RTSLibError("Can't find version for fabric module %s"
                                   % self.name)
         else:
             return None
@@ -187,7 +187,7 @@ class _BaseFabricModule(CFSNode):
         according to the spec, and returns a tuple of our preferred string 
         representation of the wwn, and what type it turned out to be.
         '''
-        return normalize_wwn(self.wwn_types, wwn, self.wwns)
+        return normalize_wwn(self.wwn_types, wwn)
 
     def to_fabric_wwn(self, wwn):
         '''
@@ -198,7 +198,10 @@ class _BaseFabricModule(CFSNode):
 
     def from_fabric_wwn(self, wwn):
         '''
-        see to_fabric_wwn.
+        Converts from WWN format used in this fabric's LIO configfs to canonical
+        format.
+        Note: Do not call from wwns(). There's no guarantee fabric wwn format is
+        the same as wherever wwns() is reading from.
         '''
         return wwn
 
@@ -211,8 +214,8 @@ class _BaseFabricModule(CFSNode):
 
     def _assert_feature(self, feature):
         if not self.has_feature(feature):
-            raise RTSLibError("This fabric module does not implement "
-                              + "the %s feature." % feature)
+            raise RTSLibError("Fabric module %s does not implement "
+                              + "the %s feature" % (self.name, feature))
 
     def clear_discovery_auth_settings(self):
         self._check_self()
@@ -347,10 +350,7 @@ class SBPFabricModule(_BaseFabricModule):
         for fname in glob("/sys/bus/firewire/devices/fw*/is_local"):
             if bool(int(fread(fname))):
                 guid_path = os.path.dirname(fname) + "/guid"
-                tmp = fread(guid_path)
-                if tmp[0:2] == '0x':
-                    tmp = tmp[2:]
-                yield self.from_fabric_wwn(tmp)
+                yield "eui." + fread(guid_path)[2:]
                 break
 
 
@@ -366,8 +366,6 @@ class Qla2xxxFabricModule(_BaseFabricModule):
         return colonize(wwn[4:])
 
     def from_fabric_wwn(self, wwn):
-        if wwn.startswith("0x"):
-            wwn = wwn[2:]
         return "naa." + wwn.replace(":", "")
 
     @property
@@ -375,7 +373,7 @@ class Qla2xxxFabricModule(_BaseFabricModule):
         for wwn_file in glob("/sys/class/fc_host/host*/port_name"):
             with ignored(IOError):
                 if not fread(os.path.dirname(wwn_file)+"/symbolic_name").startswith("fcoe"):
-                    yield self.from_fabric_wwn(fread(wwn_file))
+                    yield "naa." + fread(wwn_file)[2:]
 
 
 class SRPTFabricModule(_BaseFabricModule):
@@ -390,14 +388,12 @@ class SRPTFabricModule(_BaseFabricModule):
         return "0x" + wwn[3:]
 
     def from_fabric_wwn(self, wwn):
-        if wwn.startswith("0x"):
-            wwn = wwn[2:]
-        return "ib." + wwn.replace(":", "")
+        return "ib." + wwn[2:]
 
     @property
     def wwns(self):
         for wwn_file in glob("/sys/class/infiniband/*/ports/*/gids/0"):
-            yield self.from_fabric_wwn(fread(wwn_file))
+            yield "ib." + fread(wwn_file).replace(":", "")
 
 
 class FCoEFabricModule(_BaseFabricModule):
@@ -414,14 +410,14 @@ class FCoEFabricModule(_BaseFabricModule):
         return colonize(wwn[4:])
 
     def from_fabric_wwn(self, wwn):
-        return "naa." + wwn[2:]
+        return "naa." + wwn.replace(":", "")
 
     @property
     def wwns(self):
         for wwn_file in glob("/sys/class/fc_host/host*/port_name"):
             with ignored(IOError):
                 if fread(os.path.dirname(wwn_file)+"/symbolic_name").startswith("fcoe"):
-                    yield self.from_fabric_wwn(fread(wwn_file))
+                    yield "naa." + fread(wwn_file)[2:]
 
 
 class USBGadgetFabricModule(_BaseFabricModule):
