@@ -3,6 +3,7 @@ Implements the base CFSNode class and a few inherited variants.
 
 This file is part of RTSLib.
 Copyright (c) 2011-2013 by Datera, Inc
+Copyright (c) 2011-2014 by Red Hat, Inc.
 
 Licensed under the Apache License, Version 2.0 (the "License"); you may
 not use this file except in compliance with the License. You may obtain
@@ -32,6 +33,12 @@ class CFSNode(object):
     def __init__(self):
         self._path = self.configfs_dir
 
+    def __eq__(self, other):
+        return self._path == other._path
+
+    def __ne__(self, other):
+        return self._path != other._path
+
     def _get_path(self):
         return self._path
 
@@ -46,17 +53,17 @@ class CFSNode(object):
         if mode not in ['any', 'lookup', 'create']:
             raise RTSLibError("Invalid mode: %s" % mode)
         if self.exists and mode == 'create':
-            raise RTSLibError("This %s already exists in configFS."
+            raise RTSLibError("This %s already exists in configFS"
                               % self.__class__.__name__)
         elif not self.exists and mode == 'lookup':
-            raise RTSLibNotInCFS("No such %s in configfs: %s."
+            raise RTSLibNotInCFS("No such %s in configfs: %s"
                                  % (self.__class__.__name__, self.path))
 
         if not self.exists:
             try:
                 os.mkdir(self.path)
             except:
-                raise RTSLibError("Could not create %s in configFS."
+                raise RTSLibError("Could not create %s in configFS"
                                   % self.__class__.__name__)
 
     def _exists(self):
@@ -64,7 +71,7 @@ class CFSNode(object):
 
     def _check_self(self):
         if not self.exists:
-            raise RTSLibNotInCFS("This %s does not exist in configFS."
+            raise RTSLibNotInCFS("This %s does not exist in configFS"
                                  % self.__class__.__name__)
 
     def _list_files(self, path, writable=None):
@@ -133,15 +140,13 @@ class CFSNode(object):
         self._check_self()
         path = "%s/attrib/%s" % (self.path, str(attribute))
         if not os.path.isfile(path):
-            raise RTSLibError("Cannot find attribute: %s."
+            raise RTSLibError("Cannot find attribute: %s"
                               % str(attribute))
         else:
             try:
                 fwrite(path, "%s" % str(value))
-            except IOError, msg:
-                msg = msg[1]
-                raise RTSLibError("Cannot set attribute %s: %s"
-                                  % (str(attribute), str(msg)))
+            except Exception as e:
+                raise RTSLibError("Cannot set attribute %s: %s" % (attribute, e))
 
     def get_attribute(self, attribute):
         '''
@@ -151,8 +156,7 @@ class CFSNode(object):
         self._check_self()
         path = "%s/attrib/%s" % (self.path, str(attribute))
         if not os.path.isfile(path):
-            raise RTSLibError("Cannot find attribute: %s."
-                              % str(attribute))
+            raise RTSLibError("Cannot find attribute: %s" % attribute)
         else:
             return fread(path)
 
@@ -168,15 +172,12 @@ class CFSNode(object):
         self._check_self()
         path = "%s/param/%s" % (self.path, str(parameter))
         if not os.path.isfile(path):
-            raise RTSLibError("Cannot find parameter: %s."
-                              % str(parameter))
+            raise RTSLibError("Cannot find parameter: %s" % parameter)
         else:
             try:
                 fwrite(path, "%s\n" % str(value))
-            except IOError, msg:
-                msg = msg[1]
-                raise RTSLibError("Cannot set parameter %s: %s"
-                                  % (str(parameter), str(msg)))
+            except Exception as e:
+                raise RTSLibError("Cannot set parameter %s: %s" % (parameter, e))
 
     def get_parameter(self, parameter):
         '''
@@ -187,8 +188,7 @@ class CFSNode(object):
         self._check_self()
         path = "%s/param/%s" % (self.path, str(parameter))
         if not os.path.isfile(path):
-            raise RTSLibError("Cannot find RFC-3720 parameter: %s."
-                              % str(parameter))
+            raise RTSLibError("Cannot find RFC-3720 parameter: %s" % parameter)
         else:
             return fread(path)
 
