@@ -3,6 +3,7 @@ Implements the RTSRoot class.
 
 This file is part of RTSLib.
 Copyright (c) 2011-2013 by Datera, Inc
+Copyright (c) 2011-2014 by Red Hat, Inc.
 
 Licensed under the Apache License, Version 2.0 (the "License"); you may
 not use this file except in compliance with the License. You may obtain
@@ -25,18 +26,10 @@ import json
 from node import CFSNode
 from target import Target
 from fabric import FabricModule
-from tcm import (StorageObject, FileIOStorageObject, BlockStorageObject,
-                 PSCSIStorageObject, RDMCPStorageObject)
+from tcm import so_mapping, StorageObject
 from utils import RTSLibError, RTSLibBrokenLink, modprobe, mount_configfs
 from utils import dict_remove, set_attributes
 
-storageobjects = dict(
-    fileio=FileIOStorageObject,
-    block=BlockStorageObject,
-    pscsi=PSCSIStorageObject,
-    ramdisk=RDMCPStorageObject,
-    )
-
 default_save_file = "/etc/target/saveconfig.json"
 
 class RTSRoot(CFSNode):
@@ -98,6 +91,12 @@ class RTSRoot(CFSNode):
             for node_acl in t.node_acls:
                 yield node_acl
 
+    def _list_mapped_luns(self):
+        self._check_self()
+        for na in self.node_acls:
+            for mlun in na.mapped_luns:
+                yield mlun
+
     def _list_network_portals(self):
         self._check_self()
         for t in self.tpgs:
@@ -159,12 +158,13 @@ class RTSRoot(CFSNode):
         '''
         Takes a dict generated by dump() and reconfigures the target to match.
         Returns list of non-fatal errors that were encountered.
+        Will refuse to restore over an existing configuration unless clear_existing
+            is True.
         '''
         if clear_existing:
             self.clear_existing(confirm=True)
-        elif list(self.storage_objects) or list(self.targets):
-            raise RTSLibError("storageobjects or targets present, not restoring." +
-                              " Set clear_existing=True?")
+        elif any(self.storage_objects) or any(self.targets):
+            raise RTSLibError("storageobjects or targets present, not restoring")
 
         errors = []
 
@@ -180,7 +180,7 @@ class RTSRoot(CFSNode):
                 err_func("'name' not defined in storage object %d" % index)
                 continue
             try:
-                so_cls = storageobjects[so['plugin']]
+                so_cls = so_mapping[so['plugin']]
             except KeyError:
                 err_func("'plugin' not defined or invalid in storageobject %s" % so['name'])
                 continue
@@ -188,13 +188,15 @@ class RTSRoot(CFSNode):
             dict_remove(kwargs, ('exists', 'attributes', 'plugin', 'buffered_mode'))
             try:
                 so_obj = so_cls(**kwargs)
-            except (TypeError, ValueError):
-                err_func("Could not create StorageObject %s" % so['name'])
+            except Exception as e:
+                err_func("Could not create StorageObject %s: %s" % (so['name'], e))
                 continue
-            try:
-                set_attributes(so_obj, so.get('attributes', {}))
-            except RTSLibError:
-                err_func("Could not set an attribute for %s" % so['name'])
+
+            # Custom err func to include block name
+            def so_err_func(x):
+                return err_func("Storage Object %s/%s: %s" % (so['plugin'], so['name'], x))
+
+            set_attributes(so_obj, so.get('attributes', {}), so_err_func)
 
         # Don't need to create fabric modules
         for index, fm in enumerate(config.get('fabric_modules', [])):
@@ -259,6 +261,8 @@ class RTSRoot(CFSNode):
             doc="Get the list of all the existing TPG objects.")
     node_acls = property(_list_node_acls,
             doc="Get the list of all the existing NodeACL objects.")
+    mapped_luns = property(_list_mapped_luns,
+            doc="Get the list of all the existing MappedLUN objects.")
     sessions = property(_list_sessions,
             doc="Get the list of all the existing sessions.")
     network_portals = property(_list_network_portals,
