@@ -3,7 +3,7 @@ Implements the RTS generic Target fabric classes.
 
 This file is part of RTSLib.
 Copyright (c) 2011-2013 by Datera, Inc.
-Copyright (c) 2011-2013 by Red Hat, Inc.
+Copyright (c) 2011-2014 by Red Hat, Inc.
 
 Licensed under the Apache License, Version 2.0 (the "License"); you may
 not use this file except in compliance with the License. You may obtain
@@ -32,6 +32,8 @@ from utils import _get_auth_attr, _set_auth_attr
 import tcm
 from functools import partial
 
+auth_params = ('userid', 'password', 'mutual_userid', 'mutual_password')
+
 class Target(CFSNode):
     '''
     This is an interface to Targets in configFS.
@@ -122,8 +124,8 @@ class Target(CFSNode):
 
         try:
             t_obj = Target(fm_obj, t['wwn'])
-        except RTSLibError:
-            err_func("Could not create Target object")
+        except RTSLibError as e:
+            err_func("Could not create Target object: %s" % e)
             return
 
         for tpg in t.get('tpgs', []):
@@ -175,17 +177,17 @@ class TPG(CFSNode):
                     tag = index
                     break
             if tag is None:
-                raise RTSLibError("Cannot find an available TPG Tag.")
+                raise RTSLibError("Cannot find an available TPG Tag")
         else:
             tag = int(tag)
             if not tag > 0:
-                raise RTSLibError("The TPG Tag must be >0.")
+                raise RTSLibError("The TPG Tag must be >0")
         self._tag = tag
 
         if isinstance(parent_target, Target):
             self._parent_target = parent_target
         else:
-            raise RTSLibError("Invalid parent Target.")
+            raise RTSLibError("Invalid parent Target")
 
         self._path = "%s/tpgt_%d" % (self.parent_target.path, self.tag)
 
@@ -195,7 +197,7 @@ class TPG(CFSNode):
                 if filename.startswith("tpgt_") \
                    and os.path.isdir("%s/%s" % (target_path, filename)) \
                    and filename != "tpgt_%d" % self.tag:
-                    raise RTSLibError("Target cannot have multiple TPGs.")
+                    raise RTSLibError("Target cannot have multiple TPGs")
 
         self._create_in_cfs_ine(mode)
         if self.has_feature('nexus') and not self._get_nexus():
@@ -263,22 +265,18 @@ class TPG(CFSNode):
         self._check_self()
 
         if not self.has_feature('nexus'):
-            raise RTSLibError("The TPG does not use a nexus.")
+            raise RTSLibError("The TPG does not use a nexus")
         if self._get_nexus():
-            raise RTSLibError("The TPG's nexus initiator WWN is already set.")
-
-        # Nexus wwn type should match parent target
-        wwn_type = self.parent_target.wwn_type
-        if nexus_wwn:
-            # Not using fabric-specific version of normalize_wwn, since we
-            # want to make sure wwn conforms to regexp, but don't check
-            # against target wwn_list, since we're setting the "initiator" here.
-            nexus_wwn = normalize_wwn((wwn_type,), nexus_wwn)[0]
-        else:
-            nexus_wwn = generate_wwn(wwn_type)
+            raise RTSLibError("The TPG's nexus initiator WWN is already set")
 
         fm = self.parent_target.fabric_module
 
+        if nexus_wwn:
+            nexus_wwn = fm.to_normalized_wwn(nexus_wwn)[0]
+        else:
+            # Nexus wwn type should match parent target
+            nexus_wwn = generate_wwn(self.parent_target.wwn_type)
+
         fwrite("%s/nexus" % self.path, fm.to_fabric_wwn(nexus_wwn))
 
     def _list_node_acls(self):
@@ -404,8 +402,8 @@ class TPG(CFSNode):
     @classmethod
     def setup(cls, t_obj, tpg, err_func):
         tpg_obj = cls(t_obj, tag=tpg.get("tag", None))
-        set_attributes(tpg_obj, tpg.get('attributes', {}))
-        set_parameters(tpg_obj, tpg.get('parameters', {}))
+        set_attributes(tpg_obj, tpg.get('attributes', {}), err_func)
+        set_parameters(tpg_obj, tpg.get('parameters', {}), err_func)
 
         for lun in tpg.get('luns', []):
             LUN.setup(tpg_obj, lun, err_func)
@@ -435,7 +433,7 @@ class TPG(CFSNode):
         d['portals'] = [portal.dump() for portal in self.network_portals]
         d['node_acls'] =  [acl.dump() for acl in self.node_acls]
         if self.has_feature("auth"):
-            for attr in ("userid", "password", "mutual_userid", "mutual_password"):
+            for attr in auth_params:
                 val = getattr(self, "chap_" + attr, None)
                 if val:
                     d["chap_" + attr] = val
@@ -453,7 +451,7 @@ class LUN(CFSNode):
     # LUN private stuff
 
     def __repr__(self):
-        return "<LUN %d (%s/%s)" % (self.lun, self.storage_object.plugin,
+        return "<LUN %d (%s/%s)>" % (self.lun, self.storage_object.plugin,
                                     self.storage_object.name)
 
     def __init__(self, parent_tpg, lun=None, storage_object=None, alias=None):
@@ -484,7 +482,7 @@ class LUN(CFSNode):
         if isinstance(parent_tpg, TPG):
             self._parent_tpg = parent_tpg
         else:
-            raise RTSLibError("Invalid parent TPG.")
+            raise RTSLibError("Invalid parent TPG")
 
         if lun is None:
             luns = [l.lun for l in self.parent_tpg.luns]
@@ -505,7 +503,7 @@ class LUN(CFSNode):
 
         if storage_object is None and alias is not None:
             raise RTSLibError("The alias parameter has no meaning " \
-                              + "without the storage_object parameter.")
+                              + "without the storage_object parameter")
 
         if storage_object is not None:
             self._create_in_cfs_ine('create')
@@ -531,7 +529,7 @@ class LUN(CFSNode):
         if storage_object.exists:
             source = storage_object.path
         else:
-            raise RTSLibError("storage_object does not exist in configFS.")
+            raise RTSLibError("storage_object does not exist in configFS")
 
         os.symlink(source, destination)
 
@@ -658,7 +656,7 @@ class NetworkPortal(CFSNode):
     # NetworkPortal private stuff
 
     def __repr__(self):
-        return "<NetworkPortal %s port %s" % (self.ip_address, self.port)
+        return "<NetworkPortal %s port %s>" % (self.ip_address, self.port)
 
     def __init__(self, parent_tpg, ip_address, port=3260, mode='any'):
         '''
@@ -684,12 +682,12 @@ class NetworkPortal(CFSNode):
         try:
             self._port = int(port)
         except ValueError:
-            raise RTSLibError("Invalid port.")
+            raise RTSLibError("Invalid port")
 
         if isinstance(parent_tpg, TPG):
             self._parent_tpg = parent_tpg
         else:
-            raise RTSLibError("Invalid parent TPG.")
+            raise RTSLibError("Invalid parent TPG")
 
         self._path = "%s/np/%s:%d" \
             % (self.parent_tpg.path, self.ip_address, self.port)
@@ -697,7 +695,7 @@ class NetworkPortal(CFSNode):
         try:
             self._create_in_cfs_ine(mode)
         except OSError as msg:
-            raise RTSLibError(msg[1])
+            raise RTSLibError(msg)
 
     def _get_ip_address(self):
         return self._ip_address
@@ -751,9 +749,9 @@ class NetworkPortal(CFSNode):
         try:
             np = cls(tpg_obj, p['ip_address'], p['port'])
             np.iser = p.get('iser', False)
-        except (RTSLibError, KeyError):
-            err_func("Creating NetworkPortal object %s:%s failed" %
-                     (p['ip_address'], p['port']))
+        except (RTSLibError, KeyError) as e:
+            err_func("Creating NetworkPortal object %s:%s failed: %s" %
+                     (p['ip_address'], p['port'], e))
 
     def dump(self):
         d = super(NetworkPortal, self).dump()
@@ -795,7 +793,7 @@ class NodeACL(CFSNode):
         if isinstance(parent_tpg, TPG):
             self._parent_tpg = parent_tpg
         else:
-            raise RTSLibError("Invalid parent TPG.")
+            raise RTSLibError("Invalid parent TPG")
 
         fm = self.parent_tpg.parent_target.fabric_module
         self._node_wwn, self.wwn_type = normalize_wwn(fm.wwn_types, node_wwn)
@@ -945,11 +943,11 @@ class NodeACL(CFSNode):
             return
         try:
             acl_obj = cls(tpg_obj, acl['node_wwn'])
-        except RTSLibError:
-            err_func("Error when creating NodeACL for %s" % acl['node_wwn'])
+        except RTSLibError as e:
+            err_func("Error when creating NodeACL for %s: %s" % (acl['node_wwn'], e))
             return
 
-        set_attributes(acl_obj, acl.get('attributes', {}))
+        set_attributes(acl_obj, acl.get('attributes', {}), err_func)
 
         for mlun in acl.get('mapped_luns', []):
             MappedLUN.setup(tpg_obj, acl_obj, mlun, err_func)
@@ -970,7 +968,7 @@ class NodeACL(CFSNode):
         if self.tag:
             d['tag'] = self.tag
         if self.has_feature("auth"):
-            for attr in ("userid", "password", "mutual_userid", "mutual_password"):
+            for attr in auth_params:
                 val = getattr(self, "chap_" + attr, None)
                 if val:
                     d["chap_" + attr] = val
@@ -1019,23 +1017,23 @@ class MappedLUN(CFSNode):
 
         if not isinstance(parent_nodeacl, NodeACL):
             raise RTSLibError("The parent_nodeacl parameter must be " \
-                              + "a NodeACL object.")
+                              + "a NodeACL object")
         else:
             self._parent_nodeacl = parent_nodeacl
             if not parent_nodeacl.exists:
-                raise RTSLibError("The parent_nodeacl does not exist.")
+                raise RTSLibError("The parent_nodeacl does not exist")
 
         try:
             self._mapped_lun = int(mapped_lun)
         except ValueError:
             raise RTSLibError("The mapped_lun parameter must be an " \
-                              + "integer value.")
+                              + "integer value")
 
         self._path = "%s/lun_%d" % (self.parent_nodeacl.path, self.mapped_lun)
 
         if tpg_lun is None and write_protect is not None:
             raise RTSLibError("The write_protect parameter has no " \
-                              + "meaning without the tpg_lun parameter.")
+                              + "meaning without the tpg_lun parameter")
 
         if tpg_lun is not None:
             self._create_in_cfs_ine('create')
@@ -1056,14 +1054,14 @@ class MappedLUN(CFSNode):
                 tpg_lun = int(tpg_lun)
             except ValueError:
                 raise RTSLibError("The tpg_lun must be either an "
-                                  + "integer or a LUN object.")
+                                  + "integer or a LUN object")
         # Check that the tpg_lun exists in the TPG
         for lun in self.parent_nodeacl.parent_tpg.luns:
             if lun.lun == tpg_lun:
                 tpg_lun = lun
                 break
         if not (isinstance(tpg_lun, LUN) and tpg_lun):
-            raise RTSLibError("LUN %s does not exist in this TPG."
+            raise RTSLibError("LUN %s does not exist in this TPG"
                               % str(tpg_lun))
         os.symlink(tpg_lun.path, "%s/%s"
                    % (self.path, str(uuid.uuid4())[-10:]))
