@@ -44,7 +44,7 @@ class StorageObject(CFSNode):
         super(StorageObject, self).__init__()
         if "/" in name or " " in name or "\t" in name or "\n" in name:
             raise RTSLibError("A storage object's name cannot contain "
-                              " /, newline or spaces/tabs.")
+                              " /, newline or spaces/tabs")
         else:
             self._name = name
         self._backstore = _Backstore(name, type(self), mode)
@@ -119,7 +119,7 @@ class StorageObject(CFSNode):
             return fread(path).partition(":")[2].strip()
         else:
             raise RTSLibError("Cannot read a T10 WWN Unit Serial from "
-                              + "an unconfigured StorageObject.")
+                              + "an unconfigured StorageObject")
 
     def _set_wwn(self, wwn):
         self._check_self()
@@ -128,7 +128,7 @@ class StorageObject(CFSNode):
             fwrite(path, "%s\n" % wwn)
         else:
             raise RTSLibError("Cannot write a T10 WWN Unit Serial to "
-                              + "an unconfigured StorageObject.")
+                              + "an unconfigured StorageObject")
 
     def _set_udev_path(self, udev_path):
         self._check_self()
@@ -172,7 +172,7 @@ class StorageObject(CFSNode):
 
     def _get_status(self):
         self._check_self()
-        return self._parse_info('Status')
+        return self._parse_info('Status').lower()
 
     def _gen_attached_luns(self):
         '''
@@ -326,14 +326,14 @@ class PSCSIStorageObject(StorageObject):
                 raise RTSLibError("Cannot find SCSI device by "
                                   + "path, and dev "
                                   + "parameter not in H:C:T:L "
-                                  + "format: %s." % dev)
+                                  + "format: %s" % dev)
             else:
                 udev_path = convert_scsi_hctl_to_path(hostid,
                                                             channelid,
                                                             targetid,
                                                             lunid)
             if not udev_path:
-                raise RTSLibError("SCSI device does not exist.")
+                raise RTSLibError("SCSI device does not exist")
         else:
             udev_path = dev.strip()
 
@@ -342,7 +342,7 @@ class PSCSIStorageObject(StorageObject):
                               + "device %s (SCSI %d:%d:%d:%d) "
                               % (udev_path, hostid, channelid,
                                  targetid, lunid)
-                              + "is already in use.")
+                              + "is already in use")
 
         self._control("scsi_host_id=%d," % hostid \
                       + "scsi_channel_id=%d," % channelid \
@@ -567,10 +567,10 @@ class FileIOStorageObject(StorageObject):
         block_type = get_blockdev_type(dev)
         if block_type is None: # a file
             if os.path.exists(os.path.realpath(dev)) and not os.path.isfile(dev):
-                raise RTSLibError("Path not to a file or block device.")
+                raise RTSLibError("Path not to a file or block device")
 
             if size is None:
-                raise RTSLibError("Path is to a file, size needed.")
+                raise RTSLibError("Path is to a file, size needed")
 
             self._control("fd_dev_name=%s,fd_dev_size=%d" % (dev, size))
 
@@ -579,10 +579,10 @@ class FileIOStorageObject(StorageObject):
             # dump() saves it and thus restore() will call us with it.
 
             if block_type != 0:
-                raise RTSLibError("Device is not a TYPE_DISK block device.")
+                raise RTSLibError("Device is not a TYPE_DISK block device")
 
             if is_dev_in_use(dev):
-                raise RTSLibError("Device %s is already in use." % dev)
+                raise RTSLibError("Device %s is already in use" % dev)
 
             self._control("fd_dev_name=%s" % dev)
 
@@ -676,10 +676,10 @@ class BlockStorageObject(StorageObject):
     def _configure(self, dev, wwn, readonly):
         self._check_self()
         if get_blockdev_type(dev) != 0:
-            raise RTSLibError("Device is not a TYPE_DISK block device.")
+            raise RTSLibError("Device %s is not a TYPE_DISK block device" % dev)
         if is_dev_in_use(dev):
             raise RTSLibError("Cannot configure StorageObject because "
-                              + "device %s is already in use." % dev)
+                              + "device %s is already in use" % dev)
         self._set_udev_path(dev)
         self._control("udev_path=%s" % dev)
         self._control("readonly=%d" % readonly)
@@ -733,6 +733,92 @@ class BlockStorageObject(StorageObject):
         return d
 
 
+class UserBackedStorageObject(StorageObject):
+    '''
+    An interface to configFS storage objects for userspace-backed backstore.
+    '''
+
+    def __init__(self, name, config=None, level=None, size=None, wwn=None):
+        '''
+        @param name: The name of the UserBackedStorageObject.
+        @type name: string
+        @param dev: The path to the backend block device to be used.
+            - Example: I{dev="/dev/sda"}.
+            - The only device type that is accepted I{TYPE_DISK}.
+              For other device types, use pscsi.
+        @type dev: string
+        @param size: The size of the device to create, in bytes.
+        @type size: int
+        @param config: user-handler-specific config string.
+            - e.g. "rbd/machine1@snap4"
+        @type config: string
+        @param level: TCMU emulation level, 0 or 1. Level 0 will pass all SCSI
+              commands, 1 will just pass I/O commands, READ, WRITE, etc.
+        @type level: int
+        @return: A UserBackedStorageObject object.
+        '''
+
+        if size is not None:
+            if level is None or config is None:
+                raise RTSLibError("'size', 'level', and 'config' must be set when "
+                                  "creating a new UserBackedStorageObject")
+            if '/' not in config:
+                raise RTSLibError("'config' must contain a '/' separating subtype "
+                                  "from its configuration string")
+            super(UserBackedStorageObject, self).__init__(name, 'create')
+            try:
+                self._configure(config, level, size, wwn)
+            except:
+                self.delete()
+                raise
+        else:
+            super(UserBackedStorageObject, self).__init__(name, 'lookup')
+
+    def _configure(self, config, level, size, wwn):
+        self._check_self()
+
+        if ':' in config:
+            raise RTSLibError("':' not allowed in config string")
+        if level not in (0, 1):
+            raise RTSLibError("Current allowable levels are 0 or 1")
+        self._control("dev_config=%s" % config)
+        self._control("pass_level=%d" % level)
+        self._control("dev_size=%d" % size)
+        self._enable()
+
+        super(UserBackedStorageObject, self)._configure(wwn)
+
+    def _get_size(self):
+        self._check_self()
+        return int(self._parse_info('Size'))
+
+    def _get_level(self):
+        self._check_self()
+        return int(self._parse_info('PassLevel'))
+
+    def _get_config(self):
+        self._check_self()
+        val = self._parse_info('Config')
+        if val == "NULL":
+            return None
+        return val
+
+    size = property(_get_size,
+            doc="Get the size in bytes.")
+    level = property(_get_level,
+            doc="Get the command emulation level.")
+    config = property(_get_config,
+            doc="Get the TCMU config.")
+
+    def dump(self):
+        d = super(UserBackedStorageObject, self).dump()
+        d['wwn'] = self.wwn
+        d['size'] = self.size
+        d['level'] = self.level
+        d['config'] = self.config
+        return d
+
+
 class StorageObjectFactory(object):
     """
     Create a storage object based on a given path.
@@ -761,6 +847,7 @@ so_mapping = {
     "fileio": FileIOStorageObject,
     "iblock": BlockStorageObject,
     "block": BlockStorageObject,
+    "user": UserBackedStorageObject,
 }
 
 
@@ -769,6 +856,7 @@ bs_params = {
     RDMCPStorageObject: dict(name='ramdisk', alt_dirprefix='rd_mcp'),
     FileIOStorageObject: dict(name='fileio'),
     BlockStorageObject: dict(name='block', alt_dirprefix='iblock'),
+    UserBackedStorageObject: dict(name='user'),
     }
 
 bs_cache = {}
