@@ -128,7 +128,7 @@ def get_blockdev_size(path):
         return get_size("/sys/block/%s" % name)
     except IOError:
         # Maybe it's a partition?
-        m = re.search(r'^([a-z0-9_-!]+)(\d+)$', name)
+        m = re.search(r'^([a-z0-9_\-!]+)(\d+)$', name)
         if m:
             # If disk name ends with a digit, Linux sticks a 'p' between it and
             # the partition number in the blockdev name.
@@ -245,7 +245,7 @@ def convert_scsi_hctl_to_path(host, controller, target, lun):
         lun = int(lun)
     except ValueError:
         raise RTSLibError(
-            "The host, controller, target and lun parameter must be integers.")
+            "The host, controller, target and lun parameter must be integers")
 
     for devname in os.listdir("/sys/block"):
         path = "/dev/%s" % devname
@@ -281,11 +281,11 @@ def generate_wwn(wwn_type):
         # 5 = IEEE registered
         # 001405 = OpenIB OUI (they let us use it I guess?)
         # rest = random
-        return "naa.5001405" + uuid.uuid4().get_hex()[-9:]
+        return "naa.5001405" + uuid.uuid4().hex[-9:]
     elif wwn_type == 'eui':
-        return "eui.001405" + uuid.uuid4().get_hex()[-10:]
+        return "eui.001405" + uuid.uuid4().hex[-10:]
     else:
-        raise ValueError("Unknown WWN type: %s." % wwn_type)
+        raise ValueError("Unknown WWN type: %s" % wwn_type)
 
 def colonize(str):
     '''
@@ -378,7 +378,7 @@ def mount_configfs():
                                    stderr=subprocess.PIPE)
         (stdoutdata, stderrdata) = process.communicate()
         if process.returncode != 0:
-            raise RTSLibError("Cannot mount configfs.")
+            raise RTSLibError("Cannot mount configfs")
 
 def dict_remove(d, items):
     for item in items:
